﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace parkMonitor.view.util {
	/// <summary>
	/// 按照步骤 1a 或 1b 操作，然后执行步骤 2 以在 XAML 文件中使用此自定义控件。
	///
	/// 步骤 1a) 在当前项目中存在的 XAML 文件中使用该自定义控件。
	/// 将此 XmlNamespace 特性添加到要使用该特性的标记文件的根 
	/// 元素中: 
	///
	///     xmlns:MyNamespace="clr-namespace:parkMonitor.view.util.imageButton"
	///
	///
	/// 步骤 1b) 在其他项目中存在的 XAML 文件中使用该自定义控件。
	/// 将此 XmlNamespace 特性添加到要使用该特性的标记文件的根 
	/// 元素中: 
	///
	///     xmlns:MyNamespace="clr-namespace:parkMonitor.view.util.imageButton;assembly=parkMonitor.view.util.imageButton"
	///
	/// 您还需要添加一个从 XAML 文件所在的项目到此项目的项目引用，
	/// 并重新生成以避免编译错误: 
	///
	///     在解决方案资源管理器中右击目标项目，然后依次单击
	///     “添加引用”->“项目”->[浏览查找并选择此项目]
	///
	///
	/// 步骤 2)
	/// 继续操作并在 XAML 文件中使用控件。
	///
	///     <MyNamespace:ImageButton/>
	///
	/// </summary>
	public class ImageButton : Label {
		static ImageButton() {
			DefaultStyleKeyProperty.OverrideMetadata(typeof(ImageButton), new FrameworkPropertyMetadata(typeof(ImageButton)));
		}

		//Source
		public static readonly DependencyProperty SourcePro = DependencyProperty.Register("Source", typeof(ImageSource), typeof(ImageButton), new PropertyMetadata(null));
		public ImageSource Source {
			get { return (ImageSource)GetValue(SourcePro); }
			set { SetCurrentValue(SourcePro, value); }
		}

		//Source
		public static readonly DependencyProperty LightSourcePro = DependencyProperty.Register("LightSource", typeof(ImageSource), typeof(ImageButton), new PropertyMetadata(null));
		public ImageSource LightSource {
			get { return (ImageSource)GetValue(LightSourcePro); }
			set { SetCurrentValue(LightSourcePro, value); }
		}
	}

	class AlignCenterCvt : IValueConverter {

		public object Convert(object value, Type targetType, object parameter, CultureInfo culture) {

			double? dValue = value as double?;
			if(dValue == null) {
				return value;
			}

			return -dValue / 2;
		}

		public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture) {
			return -1;
		}

	}

}
